#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>

#define NUM_CARTAS 64
#define CARTAS_POR_LINHA 8
#define MASCARA "###"

#define JOGADA_INSUFICIENTE -3
#define JOGADA_INVALIDA     -2
#define JOGADA_REPETIDA     -1
#define JOGADA_VALIDA       1

// Gera um número pseudoaleatório a partir de uma sequência de sementes.
unsigned int randaux();

// Obtém um valor de desperdício do usuário e gera uma semente .
bool gerarSemente();

// Preenche o vetor e o embaralha baseado na semente atual.
void prepararBaralho(int baralho[], int n);

// Embaralha o vetor de cartas.
void baralhar(int baralho[], int n);

// Exibe a matriz inicial com máscara.
void exibirMatrizMascarada();

// Limpa o buffer de entrada.
void limparBuffer();

// Recebe a jogada do usuário.
int receberJogada(int jogadas[]);

// Verifica se pelo menos uma jogada está entre 11 e 88, e mascara as que não estiverem.
int validarJogada(int baralho[], int jogadas[], int n);

int realizarJogada(int baralho[], int jogadas[], int n, bool eliminar);

int realizarJogadaAutomatica(int baralho[]);

// Exibir a matriz com os valores correspondentes nas posições escolhidas pelo usuário.
int exibirMatriz(int baralho[], int jogadas[], int n, bool eliminar);

int obterIndiceCarta(int posicao_matrix);

bool compararCartas(int baralho[], int jogadas[], int n);

void mostrarResultado(int baralho[]);

int main() {
    int baralho[NUM_CARTAS];
    
    if(!gerarSemente()) {
        return 0;
    }

    prepararBaralho(baralho, NUM_CARTAS);

    exibirMatrizMascarada();

    int jogadas[4];

    realizarJogada(baralho, jogadas, NUM_CARTAS, false);

    int quantidade_jogadas = 1;
    int quantidade_cartas_removidas = 0;

    while(true) {
        int cartas_removidas = realizarJogada(baralho, jogadas, NUM_CARTAS, true);

        if(cartas_removidas == JOGADA_INVALIDA) {
            while(quantidade_cartas_removidas < NUM_CARTAS) {
                realizarJogadaAutomatica(baralho);
                quantidade_jogadas++;
                quantidade_cartas_removidas++;
            }
            break;
        }

        
        if(cartas_removidas == JOGADA_INSUFICIENTE) {
            continue;
        }

        quantidade_jogadas++;
        
        if(cartas_removidas == JOGADA_REPETIDA) {
            break;
        }

        quantidade_cartas_removidas += cartas_removidas;
    }

    mostrarResultado(baralho);

    printf("Jogo com %d jogadas. Restam %d cartas.\n", quantidade_jogadas, (NUM_CARTAS - quantidade_cartas_removidas));

    return 0;
}

char operacoes[] = {'+', '-', '*', '/'};
char numeros[] = {'1', '2', '3', '4'};
char letras[] = {'A', 'B', 'C', 'D'};

unsigned int randaux() {
    static long seed = 1;
    return (((seed = seed * 214013L + 2531011L) >> 16) & 0x7fff);
}

bool gerarSemente() {
    int desperdicio;

    scanf("%d", &desperdicio);

    limparBuffer();

    if(desperdicio < 0) {
        return false;
    }

    while(desperdicio-- > 0) {
        randaux();
    }

    return true;
}

void baralhar(int baralho[], int n) {
    for (int i = 0; i < n - 1; i++) {
        int j = i + randaux() % (n - i);
        int aux = baralho[i];
        baralho[i] = baralho[j];
        baralho[j] = aux;
    }
}

void prepararBaralho(int baralho[], int n) {
    for(int i = 0; i < n; i++) {
        baralho[i] = i;
    }

    baralhar(baralho, n);
}

void exibirMatrizMascarada() {
    printf("    [1] [2] [3] [4] [5] [6] [7] [8]\n");
    for (int i = 1; i <= CARTAS_POR_LINHA; i++) {
        printf("[%d]", i);
        for (int j = 1; j <= NUM_CARTAS / CARTAS_POR_LINHA; j++) {
            printf(" %s", MASCARA);
        }
        printf("\n");
    }
}

void limparBuffer() {
    char c;
    while((c = getchar()) != '\n' && c != EOF);
}

int receberJogada(int jogadas[]) {
    char buffer[100];

    int num_jogadas = 0;
    bool obter_entradas = true;

    int i = 0;
    do {
        i = 0;
        do {
            buffer[i++] = getchar();
        } while(buffer[i - 1] != ' ' && buffer[i - 1] != '\n');

        if(buffer[i - 1] == '\n') {
            obter_entradas = false;
        }

        buffer[i - 1] = '\0';

        jogadas[num_jogadas++] = atoi(buffer); 
        
    } while(obter_entradas && num_jogadas < 4 && buffer[i - 1] != '\n');

    return num_jogadas;
}

int validarJogada(int baralho[], int jogadas[], int n) {
    int jogadas_validas = 0;
    int trava = 0;
    for (int i = 0; i < n; i++) {
        if(trava == 0) {
            if (jogadas[i] < 11 || jogadas[i] > 88) {
                trava = 1;
            }
            else {
                jogadas_validas++;
            }
        }

        if (trava == 1) {
            jogadas[i] = -1;
        }
    }

    if(jogadas_validas == 0) {
        return JOGADA_INVALIDA;
    }

    for (int i = 1; i <= CARTAS_POR_LINHA; i++) {
        for (int j = 1; j <= NUM_CARTAS / CARTAS_POR_LINHA; j++) {

            int indice_jogada_atual = (j - 1) * CARTAS_POR_LINHA + (i - 1);
            int jogada_atual = j * 10 + i;

            for (int w = 0; w < n; w++) {
                if (jogada_atual == jogadas[w]) {
                    if (baralho[indice_jogada_atual] == -1) {
                        return JOGADA_REPETIDA;
                    }
                }
            }
        }
    }

    if(n < 2) {
        return JOGADA_INSUFICIENTE;
    }

    return JOGADA_VALIDA;
}

int realizarJogada(int baralho[], int jogadas[], int n, bool eliminar) {
    int num_jogadas = receberJogada(jogadas);
    int validade_jogada = validarJogada(baralho, jogadas, num_jogadas);
    if(validade_jogada != JOGADA_VALIDA) {
        return validade_jogada;
    }
    return exibirMatriz(baralho, jogadas, num_jogadas, eliminar);
}

int realizarJogadaAutomatica(int baralho[]) {
    // printf("    [1] [2] [3] [4] [5] [6] [7] [8]\n");
    bool jogada_realizada = false;
    for (int i = 1; i <= CARTAS_POR_LINHA; i++) { // coluna
        // printf("[%d]", i);
        for (int j = 1; j <= NUM_CARTAS / CARTAS_POR_LINHA; j++) { // linha
            int jogada_atual = i * 10 + j;
            int indice_jogada = obterIndiceCarta(jogada_atual);
            // printf("\n{INDICE: %d} -> {VALOR: %d} -> \n", indice_jogada, jogada_atual);
            if(baralho[indice_jogada] != -1) {
                // printf(" %s", MASCARA);
                if(jogada_realizada == false) {
                    baralho[indice_jogada] = -1;
                    jogada_realizada = true;
                }
            }
            else {
                // printf("    ");
            }
        }
        // printf("\n");
    }
    return 1;
}

int exibirMatriz(int baralho[], int jogadas[], int n, bool eliminar) {
    static int numero_jogadas = 1;
    
    if(numero_jogadas < 5) {
        printf("    [1] [2] [3] [4] [5] [6] [7] [8]\n");
    }

    for (int i = 1; i <= CARTAS_POR_LINHA; i++) {

        if(numero_jogadas < 5) {
            printf("[%d]", i);
        }

        for (int j = 1; j <= NUM_CARTAS / CARTAS_POR_LINHA; j++) {
            // Calcular a posição atual na matriz
            int jogada_atual = j * 10 + i;

            // Verificar se a posição atual está nas posições especificadas pelo usuário
            int revelar = 0;
            for (int x = 0; x < n; x++) {
                if (jogada_atual == jogadas[x]) {
                    revelar = 1;
                    break;
                }
            }

            int indice_jogada = obterIndiceCarta(jogada_atual);

            if(numero_jogadas < 5) {
                if (revelar) {
                    if(baralho[indice_jogada] != -1) {
                        int carta_atual = baralho[indice_jogada];
                        int k = carta_atual / 16;
                        int l = (carta_atual % 16) / 4; 
                        int m = carta_atual % 4;
                        printf(" %c%c%c", operacoes[k], numeros[l], letras[m]);
                    }
                } else {
                    if(baralho[indice_jogada] != -1) {
                        printf(" %s", MASCARA);
                    }
                    else {
                        printf("    ");
                    }
                }
            }
        }
        if(numero_jogadas < 5) {
            printf("\n");
        }
    }

    int cartas_removidas = 0;
    if(eliminar && n > 1) {
        bool jogada_certa = compararCartas(baralho, jogadas, n);

        if(jogada_certa) {
            for (int i = 0; jogada_certa && i < n; i++) {
                if(jogadas[i] == -1) {
                    break;
                }
                baralho[obterIndiceCarta(jogadas[i])] = -1;
                cartas_removidas++;
            }
        }
    }

    numero_jogadas++;
    return cartas_removidas;
}

int obterIndiceCarta(int posicao_matrix) {
    return (posicao_matrix / 10 - 1)*8 + (posicao_matrix % 10 - 1);
}

bool compararCartas(int baralho[], int jogadas[], int n) {
    int indice_jogada = obterIndiceCarta(jogadas[0]);
    int carta_atual_jogada = baralho[indice_jogada];
    int k1 = carta_atual_jogada / 16;
    int l1 = (carta_atual_jogada % 16) / 4; 
    int m1 = carta_atual_jogada % 4;

    bool todos_k_iguais = true;
    bool todos_l_iguais = true;
    bool todos_m_iguais = true;

    for(int i = 1; i < n; i++) {
        if(jogadas[i] == -1) {
            break;
        }

        int indice_carta = obterIndiceCarta(jogadas[i]);
        int carta_atual = baralho[indice_carta];
        int k2 = carta_atual / 16;
        int l2 = (carta_atual % 16) / 4; 
        int m2 = carta_atual % 4;

        if(k1 != k2) {
            todos_k_iguais = false;
        }
        if(l1 != l2) {
            todos_l_iguais = false;
        }
        if(m1 != m2) {
            todos_m_iguais = false;
        }
    }

    return (todos_k_iguais || todos_l_iguais || todos_m_iguais);           
}

void mostrarResultado(int baralho[]) {
    printf("    [1] [2] [3] [4] [5] [6] [7] [8]\n");

    for (int i = 1; i <= CARTAS_POR_LINHA; i++) {
        printf("[%d]", i);
        for (int j = 1; j <= NUM_CARTAS / CARTAS_POR_LINHA; j++) {
            // Calcular a posição atual na matriz
            int jogada_atual = j * 10 + i;

            int indice_jogada = obterIndiceCarta(jogada_atual);

            if(baralho[indice_jogada] != -1) {
                 printf(" %s", MASCARA);
            }
            else {
                printf("    ");
            }
        }
        printf("\n");
    }
}